package com.codigoparallevar.minicards.parts.connectors;

import android.util.Log;

import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.connectors.Wiring.Wire;
import com.codigoparallevar.minicards.types.connectors.input.StringInputConnector;
import com.codigoparallevar.minicards.types.wireData.StringSignal;

import java.util.LinkedList;
import java.util.List;

public class StringRoundInputConnector implements StringInputConnector {
    private final Part _part;
    private int _xposition;
    private int _yposition;
    private final int _radius;
    private final List<Wire> _attachments = new LinkedList<>();

    public StringRoundInputConnector(Part part,
                                     int inputConnectorCenterX, int inputConnectorCenterY,
                                     int inputConnectorRadius) {
        _part = part;
        _xposition = inputConnectorCenterX;
        _yposition = inputConnectorCenterY;
        _radius = inputConnectorRadius;
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return ((Math.abs(x - _xposition) <= _radius)
                && (Math.abs(y - _yposition) <= _radius));
    }

    @Override
    public Moveable getMoveable() {
        return new Wire(this, _xposition, _yposition);
    }

    @Override
    public void unlink() {
        for (Wire wire : _attachments) {
            wire.unlink();
        }
    }

    @Override
    public void updatePosition(int x, int y) {
        _xposition = x;
        _yposition = y;

        for (Wire wire : _attachments){
            wire.moveEnd(x, y);
        }
    }

    @Override
    public int getX() {
        return _xposition;
    }

    @Override
    public int getY() {
        return _yposition;
    }

    @Override
    public void send(StringSignal signal) {
        _part.send(this, signal);
    }


    @Override
    public void getAttachment(Wire<StringSignal, StringInputConnector> wire) {
        _attachments.add(wire);
    }

    @Override
    public Part getPart() {
        return _part;
    }

    @Override
    public String getId() {
        return _part.getConnectorId(this);
    }

    @Override
    public void drop(Wire wire) {
        Log.d("InputConnector", "Dropped wire " + wire);
    }

    @Override
    public void unlinkWire(Wire wire) {
        _attachments.remove(wire);
    }
}
